@@ -18,11 +18,7 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 """ Module that provides the delay class."""
-from __future__ import absolute_import
-
-from myhdl._compat import integer_types
-
-_errmsg = "arg of delay constructor should be a natural integeer"
+_errmsg = "arg of delay constructor should be a natural integer"
 
 
 class delay(object):
@@ -36,6 +32,6 @@ class delay(object):
         val -- a natural integer representing the desired delay
 
         """
-        if not isinstance(val, integer_types) or val < 0:
+        if not isinstance(val, int) or val < 0:
             raise TypeError(_errmsg)
         self._time = val
