@@ -20,10 +20,6 @@
 """ myhdl toVerilog conversion module.
 
 """
-from __future__ import absolute_import
-from __future__ import print_function
-
-
 import sys
 import math
 import os
@@ -33,14 +29,13 @@ import inspect
 from datetime import datetime
 import ast
 import string
+from io import StringIO
 
 from types import GeneratorType
-from myhdl._compat import StringIO
 import warnings
 
 import myhdl
 from myhdl import *
-from myhdl._compat import integer_types, class_types, PY2
 from myhdl import ToVerilogError, ToVerilogWarning
 from myhdl._extractHierarchy import (_HierExtr, _isMem, _getMemInfo,
                                      _UserVerilogCode, _userCodeMap)
@@ -50,7 +45,7 @@ from myhdl.conversion._misc import (_error, _kind, _context,
                                     _ConversionMixin, _Label, _genUniqueSuffix, _isConstant)
 from myhdl.conversion._analyze import (_analyzeSigs, _analyzeGens, _analyzeTopFunc,
                                        _Ram, _Rom)
-from myhdl._Signal import _Signal
+from myhdl._Signal import _Signal, Constant
 from myhdl._ShadowSignal import _TristateSignal, _TristateDriver
 
 from myhdl._block import _Block
@@ -153,7 +148,10 @@ class _ToVerilogConvertor(object):
                 _converting = 0
         else:
             warnings.warn(
-                "\n    toVerilog(): Deprecated usage: See http://dev.myhdl.org/meps/mep-114.html", stacklevel=2)
+                "\n    toVerilog(): Deprecated usage: See http://dev.myhdl.org/meps/mep-114.html",
+                stacklevel=2,
+                category=DeprecationWarning,
+            )
             try:
                 h = _HierExtr(name, func, *args, **kwargs)
             finally:
@@ -290,11 +288,6 @@ def _writeModuleHeader(f, intf, doc):
         r = _getRangeString(s)
         p = _getSignString(s)
         if s._driven:
-            if s._read:
-                if not isinstance(s, _TristateSignal):
-                    warnings.warn("%s: %s" % (_error.OutputPortRead, portname),
-                                  category=ToVerilogWarning
-                                  )
             if isinstance(s, _TristateSignal):
                 print("inout %s%s%s;" % (p, r, portname), file=f)
             else:
@@ -317,8 +310,14 @@ def _writeSigDecls(f, intf, siglist, memlist):
     for s in siglist:
         if not s._used:
             continue
+
         if s._name in intf.argnames:
             continue
+
+        if s._name.startswith('-- OpenPort'):
+            # do not write a signal declaration
+            continue
+
         r = _getRangeString(s)
         p = _getSignString(s)
         if s._driven:
@@ -342,14 +341,20 @@ def _writeSigDecls(f, intf, siglist, memlist):
                     print("%s %s%s%s = %s;" %
                           (k, p, r, s._name, _intRepr(s._init)), file=f)
         elif s._read:
-            # the original exception
-            # raise ToVerilogError(_error.UndrivenSignal, s._name)
-            # changed to a warning and a continuous assignment to a wire
-            warnings.warn("%s: %s" % (_error.UndrivenSignal, s._name),
-                          category=ToVerilogWarning
-                          )
-            constwires.append(s)
-            print("wire %s%s;" % (r, s._name), file=f)
+            if isinstance(s, Constant):
+                c = int(s.val)
+                c_len = s._nrbits
+                c_str = "%s" % c
+                print("localparam %s%s = %s'd%s;" % (r, s._name, c_len, c_str), file=f)
+            else:
+                # the original exception
+                # raise ToVerilogError(_error.UndrivenSignal, s._name)
+                # changed to a warning and a continuous assignment to a wire
+                warnings.warn("%s: %s" % (_error.UndrivenSignal, s._name),
+                              category=ToVerilogWarning
+                              )
+                constwires.append(s)
+                print("wire %s%s;" % (r, s._name), file=f)
     # print(file=f)
     for m in memlist:
         if not m._used:
@@ -394,9 +399,24 @@ def _writeSigDecls(f, intf, siglist, memlist):
                          for n, each in enumerate(m.mem)])
                     initial_assignments = (
                         'initial begin\n' + val_assignments + '\nend')
-
-        print("%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth),
-              file=f)
+            print("%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth), file=f)
+        else:
+            # remember for SystemVerilog, later
+            # # can assume it is a localparam array
+            # # build the initial values list
+            # vals = []
+            # w = m.mem[0]._nrbits
+            # for s in m.mem:
+            #     vals.append('{}\'d{}'.format(w, _intRepr(s._init)))
+            #
+            # print('localparam {} {} {} [0:{}-1] = \'{{{}}};'.format(p, r, m.name, m.depth, ', '.join(vals)), file=f)
+            print('reg {}{} {} [0:{}-1];'.format(p, r, m.name, m.depth), file=f)
+            val_assignments = '\n'.join(
+                        ['    %s[%d] <= %s;' %
+                         (m.name, n, _intRepr(each._init))
+                         for n, each in enumerate(m.mem)])
+            initial_assignments = (
+                'initial begin\n' + val_assignments + '\nend')
 
         if initial_assignments is not None:
             print(initial_assignments, file=f)
@@ -476,6 +496,7 @@ def _getSignString(s):
     else:
         return ''
 
+
 def _intRepr(n, radix=''):
     # write size for large integers (beyond 32 bits signed)
     # with some safety margin
@@ -483,18 +504,19 @@ def _intRepr(n, radix=''):
     p = abs(n)
     size = ''
     num = str(p).rstrip('L')
-    if radix == "hex" or p >= 2**30:
+    if radix == "hex" or p >= 2 ** 30:
         radix = "'h"
         num = hex(p)[2:].rstrip('L')
-    if p >= 2**30:
-        size = int(math.ceil(math.log(p+1,2))) + 1  # sign bit!
+    if p >= 2 ** 30:
+        size = int(math.ceil(math.log(p + 1, 2))) + 1  # sign bit!
 #            if not radix:
 #                radix = "'d"
     r = "%s%s%s" % (size, radix, num)
-    if n < 0: # add brackets and sign on negative numbers
+    if n < 0:  # add brackets and sign on negative numbers
         r = "(-%s)" % r
     return r
 
+
 def _convertGens(genlist, vfile):
     blockBuf = StringIO()
     funcBuf = StringIO()
@@ -755,8 +777,8 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
     def visit_Assign(self, node):
         # shortcut for expansion of ROM in case statement
         if isinstance(node.value, ast.Subscript) and \
-                isinstance(node.value.slice, ast.Index) and\
-                isinstance(node.value.value.obj, _Rom):
+                      not isinstance(node.value.slice, ast.Slice) and \
+                      isinstance(node.value.value.obj, _Rom):
             rom = node.value.value.obj.rom
 #            self.write("// synthesis parallel_case full_case")
 #            self.writeline()
@@ -835,11 +857,15 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         elif f is ord:
             opening, closing = '', ''
             node.args[0].s = str(ord(node.args[0].s))
-        elif f in integer_types:
+        elif f is int:
             opening, closing = '', ''
             # convert number argument to integer
-            if isinstance(node.args[0], ast.Num):
-                node.args[0].n = int(node.args[0].n)
+            if sys.version_info >= (3, 8, 0):
+                if isinstance(node.args[0], ast.Constant):
+                    node.args[0].n = int(node.args[0].n)
+            else:
+                if isinstance(node.args[0], ast.Num):
+                    node.args[0].n = int(node.args[0].n)
         elif f in (intbv, modbv):
             self.visit(node.args[0])
             return
@@ -851,7 +877,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(opening)
             self.visit(fn.value)
             self.write(closing)
-        elif (type(f) in class_types) and issubclass(f, Exception):
+        elif (type(f) in (type,)) and issubclass(f, Exception):
             self.write(f.__name__)
         elif f in (posedge, negedge):
             opening, closing = ' ', ''
@@ -891,20 +917,49 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.write(")")
         self.context = None
 
-    def visit_Num(self, node):
-        if self.context == _context.PRINT:
-            self.write('"%s"' % node.n)
-        else:
-            self.write(self.IntRepr(node.n))
+    if sys.version_info >= (3, 9, 0):
 
-    def visit_Str(self, node):
-        s = node.s
-        if self.context == _context.PRINT:
-            self.write('"%s"' % s)
-        elif len(s) == s.count('0') + s.count('1'):
-            self.write("%s'b%s" % (len(s), s))
-        else:
-            self.write(s)
+        def visit_Constant(self, node):
+            if node.value is None:
+                # NameConstant
+                self.write(nameconstant_map[node.obj])
+            elif isinstance(node.value, bool):
+                self.write(nameconstant_map[node.obj])
+            elif isinstance(node.value, int):
+                # Num
+                if self.context == _context.PRINT:
+                    self.write('"%s"' % node.value)
+                else:
+                    self.write(self.IntRepr(node.value))
+            elif isinstance(node.value, str):
+                # Str
+                s = node.value
+                if self.context == _context.PRINT:
+                    self.write('"%s"' % s)
+                elif len(s) == s.count('0') + s.count('1'):
+                    self.write("%s'b%s" % (len(s), s))
+                else:
+                    self.write(s)
+
+    else:
+
+        def visit_Num(self, node):
+            if self.context == _context.PRINT:
+                self.write('"%s"' % node.n)
+            else:
+                self.write(self.IntRepr(node.n))
+
+        def visit_Str(self, node):
+            s = node.s
+            if self.context == _context.PRINT:
+                self.write('"%s"' % s)
+            elif len(s) == s.count('0') + s.count('1'):
+                self.write("%s'b%s" % (len(s), s))
+            else:
+                self.write(s)
+
+        def visit_NameConstant(self, node):
+            self.write(nameconstant_map[node.obj])
 
     def visit_Continue(self, node):
         self.write("disable %s;" % self.labelStack[-1])
@@ -917,8 +972,12 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(doc)
             return
         # skip extra semicolons
-        if isinstance(expr, ast.Num):
-            return
+        if sys.version_info >= (3, 8, 0):
+            if isinstance(expr, ast.Constant):
+                return
+        else:
+            if isinstance(expr, ast.Num):
+                return
         self.visit(expr)
         # ugly hack to detect an orphan "task" call
         if isinstance(expr, ast.Call) and hasattr(expr, 'tree'):
@@ -1004,17 +1063,79 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         else:
             self.mapToIf(node)
 
+    def visit_Match(self, node):
+        self.write("case (")
+        self.visit(node.subject)
+        self.write(")")
+        self.indent()
+        for case in node.cases:
+            self.visit(case)
+            self.writeline()
+
+        self.dedent()
+        self.writeline()
+        self.write("endcase")
+
+    def visit_match_case(self, node):
+        pattern = node.pattern
+        self.visit(pattern)
+
+        self.write(": begin ")
+        self.indent()
+        # Write all the multiple assignment per case
+        for stmt in node.body:
+            self.writeline()
+            self.visit(stmt)
+        self.dedent()
+        self.writeline()
+        self.write("end")
+
+    def visit_MatchValue(self, node):
+        item = node.value
+        obj = self.getObj(item)
+
+        if isinstance(obj, EnumItemType):
+            itemRepr = obj._toVerilog()
+        else:
+            itemRepr = self.IntRepr(item.value, radix='hex')
+
+        self.write(itemRepr)
+
+    def visit_MatchSingleton(self, node):
+        raise AssertionError("Unsupported Match type %s " % (type(node)))
+
+    def visit_MatchSequence(self, node):
+        raise AssertionError("Unsupported Match type %s " % (type(node)))
+
+    def visit_MatchStar(self, node):
+        raise AssertionError("Unsupported Match type %s " % (type(node)))
+
+    def visit_MatchMapping(self, node):
+        raise AssertionError("Unsupported Match type %s " % (type(node)))
+
+    def visit_MatchClass(self, node):
+        for pattern in node.patterns:
+            self.visit(pattern)
+
+    def visit_MatchAs(self, node):
+        if node.name is None and  node.pattern is None:
+            self.write("default")
+        else:
+            raise AssertionError("Unknown name %s or pattern %s" % (node.name, node.pattern))
+
+    def visit_MatchOr(self, node):
+        for i, pattern in enumerate(node.patterns):
+            self.visit(pattern)
+            if not i == len(node.patterns) - 1:
+                self.write(" | ")
+
     def mapToCase(self, node, *args):
         var = node.caseVar
 #        self.write("// synthesis parallel_case")
 #        if node.isFullCase:
 #            self.write(" full_case")
 #        self.writeline()
-        caseType = "case"
-        if isinstance(node.caseItem, EnumItemType):
-            if node.caseItem._type._encoding in ('one_hot', 'one_cold'):
-                caseType = "casez"
-        self.write("%s (" % caseType)
+        self.write("case (")
         self.visit(var)
         self.write(")")
         self.indent()
@@ -1022,7 +1143,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.writeline()
             item = test.case[1]
             if isinstance(item, EnumItemType):
-                self.write(item._toVerilog(dontcare=True))
+                self.write(item._toVerilog())
             else:
                 self.write(self.IntRepr(item, radix='hex'))
             self.write(": begin")
@@ -1077,10 +1198,8 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.visit(stmt)
 
     def visit_ListComp(self, node):
-        pass  # do nothing
-
-    def visit_NameConstant(self, node):
-        self.write(nameconstant_map[node.obj])
+        # do nothing
+        pass
 
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
@@ -1093,10 +1212,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def getName(self, node):
         n = node.id
-        if PY2 and n in ('True', 'False', 'None'):
-            self.visit_NameConstant(node)
-            return
-
         addSignBit = False
         isMixedExpr = (not node.signed) and (self.context == _context.SIGNED)
         if n in self.tree.vardict:
@@ -1110,8 +1225,10 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             obj = self.tree.symdict[n]
             if isinstance(obj, bool):
                 s = "1'b%s" % int(obj)
-            elif isinstance(obj, integer_types):
+            elif isinstance(obj, int):
                 s = self.IntRepr(obj)
+            elif isinstance(obj, tuple):  # Python3.9+ ast.Index replacement serves a tuple
+                s = n
             elif isinstance(obj, _Signal):
                 addSignBit = isMixedExpr
                 s = str(obj)
@@ -1121,15 +1238,19 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 s = m.name
             elif isinstance(obj, EnumItemType):
                 s = obj._toVerilog()
-            elif (type(obj) in class_types) and issubclass(obj, Exception):
+            elif (type(obj) in (type,)) and issubclass(obj, Exception):
                 s = n
             else:
-                self.raiseError(node, _error.UnsupportedType, "%s, %s" % (n, type(obj)))
+                self.raiseError(node, _error.UnsupportedType, "%s, %s %s" % (n, type(obj), obj))
         else:
             raise AssertionError("name ref: %s" % n)
         if addSignBit:
             self.write("$signed({1'b0, ")
-        self.write(s)
+
+        if s.startswith('--'):
+            self.write(s.replace('--', '//'))
+        else:
+            self.write(s)
         if addSignBit:
             self.write("})")
 
@@ -1179,7 +1300,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                     self.writeline()
                     self.write("endcase")
                 else:
-                    print (type(obj), type(a))
                     self.write('$write("%s", ' % fs)
                     self.visit(a)
                     self.write(');')
@@ -1216,6 +1336,15 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         # special shortcut case for [:] slice
         if lower is None and upper is None:
             return
+
+        if isinstance(lower, ast.BinOp) and isinstance(lower.left, ast.Name) and isinstance(upper, ast.Name) and upper.id == lower.left.id and isinstance(lower.op, ast.Add):
+            self.write("[")
+            self.visit(upper)
+            self.write("+:")
+            self.visit(lower.right)
+            self.write("]")
+            return
+
         self.write("[")
         if lower is None:
             self.write("%s" % node.obj._nrbits)
@@ -1240,7 +1369,10 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.visit(node.value)
         self.write("[")
         # assert len(node.subs) == 1
-        self.visit(node.slice.value)
+        if sys.version_info >= (3, 9, 0):  # Python 3.9+: no ast.Index wrapper
+            self.visit(node.slice)
+        else:
+            self.visit(node.slice.value)
         self.write("]")
         if addSignBit:
             self.write("})")
@@ -1369,6 +1501,12 @@ class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
 
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
+            # try intercepting '-- OpenPort' signals
+            if isinstance(node.value, ast.Name):
+                obj = self.tree.symdict[node.value.id]
+                if obj._name.startswith('-- OpenPort'):
+                    self.write('// ')
+
             self.write("assign ")
             self.visit(node.value)
         else:
@@ -1406,7 +1544,7 @@ def _convertInitVal(reg, init):
     if tipe is bool:
         v = '1' if init else '0'
     elif tipe is intbv:
-        init = int(init) # int representation
+        init = int(init)  # int representation
         v = "%s" % init if init is not None else "'bz"
     else:
         assert isinstance(init, EnumItemType)
@@ -1535,7 +1673,7 @@ class _ConvertTaskVisitor(_ConvertVisitor):
 def _maybeNegative(obj):
     if hasattr(obj, '_min') and (obj._min is not None) and (obj._min < 0):
         return True
-    if isinstance(obj, integer_types) and obj < 0:
+    if isinstance(obj, int) and obj < 0:
         return True
     return False
 
@@ -1568,8 +1706,12 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
         node.signed = node.operand.signed
         if isinstance(node.op, ast.USub):
             node.obj = int(-1)
-            if isinstance(node.operand, ast.Num):
-                node.signed = True
+            if sys.version_info >= (3, 8, 0):
+                if isinstance(node.operand, ast.Constant):
+                    node.signed = True
+            else:
+                if isinstance(node.operand, ast.Num):
+                    node.signed = True
 
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
@@ -1609,11 +1751,21 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             return
         self.generic_visit(node)
 
-    def visit_Num(self, node):
-        node.signed = False
+    if sys.version_info >= (3, 9, 0):
 
-    def visit_Str(self, node):
-        node.signed = False
+        def visit_Constant(self, node):
+            node.signed = False
+
+    else:
+
+        def visit_Num(self, node):
+            node.signed = False
+
+        def visit_Str(self, node):
+            node.signed = False
+
+        def visit_NameConstant(self, node):
+            node.signed = False
 
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
