@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
 import sys
 import os
 import tempfile
@@ -17,7 +15,7 @@ from myhdl._block import _Block
 
 _version = myhdl.__version__.replace('.', '')
 # strip 'dev' for version
-_version = _version.replace('dev', '')
+_version = _version.replace('dev', '')[:-2]
 
 _simulators = {}
 
@@ -40,12 +38,13 @@ def registerSimulator(name=None, hdl=None, analyze=None, elaborate=None, simulat
         raise ValueError("Invalid simulator command")
     _simulators[name] = sim(name, hdl, analyze, elaborate, simulate, skiplines, skipchars, ignore)
 
+
 registerSimulator(
     name="ghdl",
     hdl="VHDL",
     analyze="ghdl -a --std=08 --workdir=work pck_myhdl_%(version)s.vhd %(topname)s.vhd",
     elaborate="ghdl -e --std=08 --workdir=work %(unitname)s",
-    simulate="ghdl -r --workdir=work %(unitname)s"
+    simulate="ghdl -r --std=08 --workdir=work %(unitname)s --ieee-asserts=disable"
 )
 
 registerSimulator(
@@ -76,7 +75,6 @@ registerSimulator(
     ignore=("# **", "# //", "#    Time:", "# run -all")
 )
 
-
 registerSimulator(
     name="iverilog",
     hdl="Verilog",
@@ -98,7 +96,7 @@ class _VerificationClass(object):
     __slots__ = ("simulator", "_analyzeOnly")
 
     def __init__(self, analyzeOnly=False):
-        self.simulator = None
+        self.simulator = 'ghdl'
         self._analyzeOnly = analyzeOnly
 
     def __call__(self, func, *args, **kwargs):
@@ -117,7 +115,10 @@ class _VerificationClass(object):
             name = func.func.__name__
         else:
             warnings.warn(
-                "\n    analyze()/verify(): Deprecated usage: See http://dev.myhdl.org/meps/mep-114.html", stacklevel=2)
+                "\n    analyze()/verify(): Deprecated usage: See http://dev.myhdl.org/meps/mep-114.html",
+                stacklevel=2,
+                category=DeprecationWarning,
+            )
             try:
                 name = func.__name__
             except:
@@ -139,9 +140,9 @@ class _VerificationClass(object):
 
         if isinstance(func, _Block):
             if hdl == "VHDL":
-                inst = func.convert(hdl='VHDL')
+                inst = func.convert(hdl='VHDL', **kwargs)
             else:
-                inst = func.convert(hdl='Verilog')
+                inst = func.convert(hdl='Verilog', **kwargs)
         else:
             if hdl == "VHDL":
                 inst = toVHDL(func, *args, **kwargs)
